#!/bin/env python
"""
 CC128 RRD Logger v0.2
=======================
Author: Paul Banks (http://paulbanks.org/)

Continuously logs data from CC128 monitor to an RRD.

Notes:

    * You might want to adjust the RRD database rrdconfig variable first.
    * You might need to set RRDTool location at the top of this file.

Usage:

CC2Logger.py -s serialport -o outputdir [-d] [-R diskfile] [-D date] [-t]
  
    -s serialport         Set serial port device, (e.g. /dev/ttyUSB0)
    -o outputdir          Set directory for output files (e.g. data/)
    -d                    Run in background (fork/daemonise)
    -R diskfile           Initialise database with contents of diskfile
    -D date               Start date of diskfile in YYYYMMDD format

"""
# If you have the python RRDTool module installed in a non-site place
# Uncomment below and update path
import sys
#sys.path.append('/opt/rrdtool-1.4.4/lib/python2.6/site-packages/')

import os
import getopt
from time import mktime
from datetime import datetime, timedelta, time

from CC2SerialMsg import *

# These imports usually require something else installed
try:
    import serial
except ImportError:
    print "Failed to import python serial (PySerial) module. Is it installed?"
    sys.exit(3)    
try:
    import rrdtool
except ImportError:
    print """    Failed to import python RRDTool module. Is it installed?
    You may need to update the path if installed in a non-standard place."""
    sys.exit(3)


def CreateRRD(filename, datadef="DS:chdata:GAUGE:60:0:U"):
    rrdtool.create(
        filename, 
        "-s", "10",
        "-b", "now-365d",
        datadef,
        "RRA:AVERAGE:0:1:17280",     # 120 days, 10 second resolution
        "RRA:AVERAGE:0.5:30:210240", # 2 years, 5 minute resolutuion
    )           


def ProcessData(outputdir, data, overridedate=None):
    # RRD Timestamp
    rrdts = "N:"
    if overridedate:
        rrdts = "%d:" % mktime(
                    datetime.combine(overridedate, data.time).timetuple()
                )

    # Update current usage files
    for ch in data.ch:
        nowfilename = outputdir+"/now_%s_ch%d.txt" % (data.sensor, ch)
        with open(nowfilename+".tmp", "w") as f:
            f.write(str(data.ch[ch]))
        os.rename(nowfilename+".tmp", nowfilename)

    # Update temperature RRDs if sensor = 0
    if data.sensor=="0":
        if not os.path.exists(outputdir+"temperature.rrd"):
            CreateRRD(outputdir+"/temperature.rrd", "DS:temp:GAUGE:60:-200:200") 
        try:
            rrdpoint = "%s%f" % (rrdts, data.tmpr)
            rrdtool.update(outputdir+"/temperature.rrd", rrdpoint)
        except rrdtool.error:
            print "Error updating temperature"

    # Update channel RRDs
    for ch in data.ch:
        rrdfilename = outputdir+"/rrd_%s_ch%d.rrd" % (str(data.sensor), ch);
        if not os.path.exists(rrdfilename):
            CreateRRD(rrdfilename)
        try:
            rrdpoint = "%s%d" % (rrdts, data.ch[ch])
            rrdtool.update(rrdfilename, rrdpoint)        
        except rrdtool.error:
            print "Error updating %s at %s" % (rrdfilename, rrdts)

def Usage():
    print __doc__

def Main():

    # Get command line args
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hds:R:o:D:w:")
    except getopt.GetoptError, err:
        print str(err)
        Usage()
        sys.exit(2)

    # Process args
    serialport = None
    outdir = None
    currentwattsfile = None
    daemonise = False
    diskfile = None
    overridedate = None
    datalogfile = None
    for o, a in opts:
        if o == "-h":
            Usage()
            sys.exit(0)         
        if o == "-s":
            serialport = a
        elif o == "-o":
            outdir = a
        elif o == "-d":
            daemonise = True
        elif o == "-R":
            if overridedate==None:
                overridedate = datetime.now()
            diskfile = a
        elif o == "-D":
            try:
                overridedate = datetime.strptime(a, "%Y%m%d")
            except ValueError:
                print "Invalid date."
                sys.exit(2)
        elif o == "-w":
            try:
                datalogfile = open(a, "a")
            except:
                print "Could not open log file"
                sys.exit(2)

    # Check we have enough information to start
    if serialport==None and diskfile==None:
        print "You must specify a serial port or filename! e.g. -s /dev/ttyUSB0"
        sys.exit(2)
    if serialport!=None and diskfile!=None:
        print "Specify disk file or serial port but not both."
        sys.exit(2)

    if outdir==None:
        print "You must specify an output directory. Use -h for help."
        sys.exit(2)

    # Open serial port or disk file
    if serialport != None:
        s = serial.Serial(serialport, 57600)
    else:
        s = open(diskfile)

    if daemonise:
        pid = os.fork()
        if pid:
            sys.exit()

    # Loop
    lastmsgtime = time(0) 
    while 1:
        try:
            # Read line of data from CC128 device or file
            l = s.readline()
            if len(l)==0:
                break

            # Log data if required
            if datalogfile != None:
                datalogfile.write(l)
                datalogfile.flush()

            # Create CC2SerialMsg object
            msg = CC2SerialMsg(l)

            # If object has required data, process it
            if hasattr(msg, "ch"):

                # Reconstruct date if using log file
                if overridedate!=None:
                    if msg.time.hour < lastmsgtime.hour:
                        overridedate += timedelta(1)                
                        if not daemonise:
                            print "Now processing", overridedate
                lastmsgtime = msg.time

                # Process message
                ProcessData(outdir, msg, overridedate)

        except CC2SerialMsgInvalid:
            continue

    if not daemonise:
        print "Program ended."
        
if __name__=="__main__":
    Main()

