#!/bin/env python
""" 
 CC128 XML to Python object converter
======================================
Author: Paul Banks (http://paulbanks.org/)

This module provides the CC2SerialMsg class.

The CC2SerialMsg class is initialised using an XML string as sent from the 
CC128 monitor device.

The resulting object then has the following properties:

    src - Type and version of the monitor (string)
    dsb - Days-since-birth or uptime in days (string)
    sensor - Sensor ID from which the attatched data came (string)
    id - Radio ID of the sensor (string)
    type - Type of sensor 1=Electricity (string)
    time - Time of the message, as specified by the monitor (time object)
    tmpr - Temperature (float)
    ch[n] - A dictionary containing channel data in watts. (int{})
            First channel is ch[1]

Exceptions:

    If the message cannot be parsed, a CC2SerialMsgInvalid is raised.

Limitations:

    No History data parsing. I don't care about this data anyway as this is
    already a continuous logging application.

    If you pass a history message, various attributes will not be set on the 
    returned object so you should check the returned object to make sure it 
    has the required required attributes before use.
      
"""

from xml.dom.minidom import parseString
from xml.parsers.expat import ExpatError
from datetime import datetime
import re

class CC2SerialMsgInvalid(Exception):
    pass

class CC2SerialMsg:

    def __init__(self, msg):
        self.Parse(msg)

    def Parse(self, msg):
        try:
            d = parseString(msg)
        except ExpatError:
            raise CC2SerialMsgInvalid()

        if d.documentElement.tagName!="msg":
            raise RuntimeError("Invalid message")

        msgroottypes = ["src", "dsb", "sensor", "id", "type"]

        channelre = re.compile("ch([0-9]+)[\s]*$");

        for n in d.childNodes[0].childNodes:

            # Get main types
            for t in msgroottypes:
                if n.tagName==t:
                    setattr(self, t, n.childNodes[0].data)
                    continue

            # Get time as a python time object
            if n.tagName=="time":
                t = datetime.strptime(n.childNodes[0].data, "%H:%M:%S").time()
                setattr(self, n.tagName, t)
                continue

            # Get temperature as float
            if n.tagName=="tmpr":
                setattr(self, n.tagName, float(n.childNodes[0].data))
                continue

            # Get channel information
            mch = channelre.match(n.tagName)
            if mch != None:
                if not hasattr(self, "ch"):
                    self.ch = {}
                channel = mch.group(1)
                self.ch[int(channel)] = \
                    int(n.getElementsByTagName("watts")[0].childNodes[0].data)
                continue
           
            #TODO: Parse history data
            # (actually, I don't care about this data...)

    def __repr__(self):
        return "CC2SerialMsg<%s, %s, ...>" % (self.src, self.time)

    pass


if __name__ == "__main__":
    f = open("testdata.txt", "r")
    msg = None
    for l in f:
       try:
            msg = CC2SerialMsg(l)
            if hasattr(msg, "ch"):
                print msg.ch[1], msg.tmpr
       except:
            print "Skipped due to error.", l


